﻿using CallCenter.Application.Handlers;
using CallCenter.Calls;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using System.Net.Http;
using System.Text;
using System.Text.Json;
using System.Threading;

namespace CallCenter.Application.CacheManager
{
    public class CallRecordFailManager : BackgroundService
    {
        private readonly IServiceScopeFactory _serviceScopeFactory;

        public CallRecordFailManager(IServiceScopeFactory serviceScopeFactory)
        {
            _serviceScopeFactory = serviceScopeFactory;
        }


        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            using var sc = _serviceScopeFactory.CreateScope();
            var _logger = sc.ServiceProvider.GetService<ILogger<CallRecordFailManager>>();
            var time = TimeSpan.FromMinutes(1);
            while (!stoppingToken.IsCancellationRequested)
            {
                using var scope = _serviceScopeFactory.CreateScope();
                var _callRecordFailRepository = scope.ServiceProvider.GetService<ICallRecordFailRepository>();
                var list = await _callRecordFailRepository.QueryAsync();
                if (list.Count > 0)
                {
                    var _sendCallRecordOptions = _serviceScopeFactory.CreateScope().ServiceProvider.GetRequiredService<IOptionsSnapshot<SendCallRecord>>();
                    var _httpClientFactory = scope.ServiceProvider.GetService<IHttpClientFactory>();
                    foreach (var item in list)
                    {
                        try
                        {
                            using var client = _httpClientFactory.CreateClient();
                            client.DefaultRequestHeaders.ConnectionClose = true;
                            var content = new StringContent(item.FailString, Encoding.UTF8, "application/json");
                            HttpResponseMessage responseMessage;
                            switch (item.FailType)
                            {
                                case Share.Enums.EFailType.CallRecord:
                                    responseMessage = await client.PostAsync(_sendCallRecordOptions.Value.FwUrl, content);
                                    break;
                                case Share.Enums.EFailType.NotReceived:
                                    responseMessage = await client.PostAsync(_sendCallRecordOptions.Value.NotReceivedUrl, content);
                                    break;
                                default:
                                    responseMessage = await client.PostAsync(_sendCallRecordOptions.Value.FwUrl, content);
                                    break;
                            }
                            var respContent = responseMessage.Content;
                            var respContentString = await respContent.ReadAsStringAsync();
                            var result = JsonSerializer.Deserialize<FwResult>(respContentString);
                            switch (item.FailType)
                            {
                                case Share.Enums.EFailType.CallRecord:
                                    _logger.LogInformation("推送通话报告结果：" + respContentString);
                                    break;
                                case Share.Enums.EFailType.NotReceived:
                                    _logger.LogInformation("推送未接听报告结果：" + respContentString);
                                    break;
                                default:
                                    break;
                            }
                            //推送成功就删除数据
                            await _callRecordFailRepository.RemoveAsync(item);
                        }
                        catch (Exception ex)
                        {
                            _logger.LogInformation("推送通话报告失败：" + ex.Message + " 错误ID:" + item.Id);
                        }
                    }
                }

                await Task.Delay(time, stoppingToken);
            }
        }
    }
}
